#include <windows.h>
#include <math.h>
#include <gl\gl.h>                      
#include <gl\glu.h>    
#include "world.h"

c_editor::c_editor() {

	this->x = 0.0;
	this->y = 0.0;

	this->lx = 0;
	this->ly = 0;

	this->xa = 45.0;
	this->ya = 0.0;

	this->ws = 0.0;
	this->ss = 0.0;

	this->xas = 0.0;
	this->yas = 0.0;

	this->camera = 40.0;

	this->mode = 0;
	this->rotmode = 1;
	this->cmod = 0.0;

}

c_editor::~c_editor() { }

float c_editor::wg2r (float angle) { // Gradus to Radian
	if (angle > 360.0)	angle -= 360.0;
	if (angle < 0.0)	angle += 360.0;
	return angle * 3.141 * 2.0 / 360.0;
}

void c_editor::process(void *vworld) {

	c_world_p world = (c_world_p) vworld;

	if (world->keys[VK_F1])		this->mode = 1;
	if (world->keys[VK_F2])		this->mode = 2;
	if (world->keys[VK_F3])		this->mode = 3;
	if (world->keys[VK_F4])		this->mode = 4;
	if (world->keys[VK_F5])		this->mode = 0;
	if (world->keys[VK_F6])		this->mode = 5;

	int md, tx, tx_id, tx_rt;

	switch (mode) {

		case 0:

			if (this->rotmode == 0) {

				if ( world->keys[VK_UP] )	{
					this->xas += (XROTATE_SPEED / 10.0f);
					if (this->xas > XROTATE_SPEED) this->xas = XROTATE_SPEED;
				}

				if ( world->keys[VK_DOWN] )  {
					this->xas -= (XROTATE_SPEED / 10.0f);
					if (this->xas < -XROTATE_SPEED) this->xas = -XROTATE_SPEED;
				}
			
				if ( world->keys[VK_RIGHT] )	{
					this->yas += (YROTATE_SPEED / 10.0f);
					if (this->yas > YROTATE_SPEED) this->yas = YROTATE_SPEED;
				}
			
				if ( world->keys[VK_LEFT] )  {
					this->yas -= (YROTATE_SPEED / 10.0f);
					if (this->yas < -YROTATE_SPEED) this->yas = -YROTATE_SPEED;
				}

			}

			if (world->keys[VK_SPACE] && !world->keyp[VK_SPACE]) {
				this->rotmode = ! this->rotmode;
				world->keyp[VK_SPACE] = true;
			}
			if (!world->keys[VK_SPACE] && world->keyp[VK_SPACE]) world->keyp[VK_SPACE] = false;

			if (world->keys[VK_NEXT])	this->camera += 1.5;
			if (world->keys[VK_PRIOR])	this->camera -= 1.5;
			if (this->camera < 10.0)	this->camera = 10.0;
			if (this->camera > 70.0)	this->camera = 70.0;

			break;

		case 1:

			if (world->keys[VK_SPACE] && !world->keyp[VK_SPACE]) {
				int l = this->get_map_land(this->lx, this->ly);
				l = (l == 1) ? 0 : 1;
				this->set_map_land (this->lx, this->ly, l);
				world->keyp[VK_SPACE] = true;
			}
			if (!world->keys[VK_SPACE] && world->keyp[VK_SPACE]) world->keyp[VK_SPACE] = false;

			break;

		case 2:

			if (world->keys[VK_PRIOR]) {
				int h = this->get_map_height_i(this->lx, this->ly);
				h ++;
				this->set_map_height_i(this->lx, this->ly, h);
			}

			if (world->keys[VK_NEXT]) {
				int h = this->get_map_height_i(this->lx, this->ly);
				h --;
				this->set_map_height_i(this->lx, this->ly, h);
			}

			if (world->keys[VK_HOME]) {
				this->set_map_height_i(this->lx, this->ly, 255);
			}

			if (world->keys[VK_END]) {
				this->set_map_height_i(this->lx, this->ly, 0);
			}

			if (world->keys['0']) this->set_map_height_i(this->lx, this->ly,   0);
			if (world->keys['1']) this->set_map_height_i(this->lx, this->ly,  20);
			if (world->keys['2']) this->set_map_height_i(this->lx, this->ly,  40);
			if (world->keys['3']) this->set_map_height_i(this->lx, this->ly,  60);
			if (world->keys['4']) this->set_map_height_i(this->lx, this->ly,  80);
			if (world->keys['5']) this->set_map_height_i(this->lx, this->ly, 100);
			if (world->keys['6']) this->set_map_height_i(this->lx, this->ly, 120);
			if (world->keys['7']) this->set_map_height_i(this->lx, this->ly, 140);
			if (world->keys['8']) this->set_map_height_i(this->lx, this->ly, 160);
			if (world->keys['9']) this->set_map_height_i(this->lx, this->ly, 180);

			break;

		case 3:

			if (world->keys[VK_SPACE] && !world->keyp[VK_SPACE]) {
				int r = this->get_map_recource(this->lx, this->ly);
				r = (r) ? 0 : 1;
				this->set_map_recource (this->lx, this->ly, r);
				world->keyp[VK_SPACE] = true;
			}
			if (!world->keys[VK_SPACE] && world->keyp[VK_SPACE]) world->keyp[VK_SPACE] = false;

			break;

		case 4:

			tx = this->get_map_texture(this->lx, this->ly);
			tx_id = map_tex_id (tx);
			tx_rt = map_tex_rt (tx);

			if (world->keys[VK_PRIOR] && !world->keyp[VK_PRIOR]) {
				tx_id = (tx_id + 1) % 64;				
				this->set_map_texture(this->lx, this->ly, tx_id, tx_rt);
				world->keyp[VK_PRIOR] = true;
			}
			if (!world->keys[VK_PRIOR]) world->keyp[VK_PRIOR] = false;

			if (world->keys[VK_NEXT] && !world->keyp[VK_NEXT]) {
				tx_id = (tx_id + 63) % 64;				
				this->set_map_texture(this->lx, this->ly, tx_id, tx_rt);
				world->keyp[VK_NEXT] = true;
			}
			if (!world->keys[VK_NEXT]) world->keyp[VK_NEXT] = false;

			if (world->keys[VK_HOME] && !world->keyp[VK_HOME]) {
				tx_rt = (tx_rt + 1) % 4;				
				this->set_map_texture(this->lx, this->ly, tx_id, tx_rt);
				world->keyp[VK_HOME] = true;
			}
			if (!world->keys[VK_HOME]) world->keyp[VK_HOME] = false;

			if (world->keys[VK_END] && !world->keyp[VK_END]) {
				tx_rt = (tx_rt + 3) % 4;				
				this->set_map_texture(this->lx, this->ly, tx_id, tx_rt);
				world->keyp[VK_END] = true;
			}
			if (!world->keys[VK_END]) world->keyp[VK_END] = false;

			break;

		case 5:

			md = 5;
			if (world->keys[VK_CONTROL]) md = 1;
			if (world->keys[VK_SHIFT]) md = 10;

			if (world->keys[VK_UP] && !world->keyp[VK_UP]) {
				int ny = this->map_y - md;
				if (ny < 15) ny = 15;
				this->resize (this->map_x, ny);
				world->keyp[VK_UP] = true;
			}
			if (!world->keys[VK_UP]) world->keyp[VK_UP] = false;

			if (world->keys[VK_DOWN] && !world->keyp[VK_DOWN]) {
				int ny = this->map_y + md;
				if (ny > 500) ny = 500;
				this->resize (this->map_x, ny);
				world->keyp[VK_DOWN] = true;
			}
			if (!world->keys[VK_DOWN]) world->keyp[VK_DOWN] = false;

			if (world->keys[VK_LEFT] && !world->keyp[VK_LEFT]) {
				int nx = this->map_x - md;
				if (nx < 15) nx = 15;
				this->resize (nx, this->map_y);
				world->keyp[VK_LEFT] = true;
			}
			if (!world->keys[VK_LEFT]) world->keyp[VK_LEFT] = false;

			if (world->keys[VK_RIGHT] && !world->keyp[VK_RIGHT]) {
				int nx = this->map_x + md;
				if (nx > 500) nx = 500;
				this->resize (nx, this->map_y);
				world->keyp[VK_RIGHT] = true;
			}
			if (!world->keys[VK_RIGHT]) world->keyp[VK_RIGHT] = false;

			break;

	}

	int xy_mod[8][4][2] = {
		{{0,-1},{0,1},{-1,0},{1,0}},{{-1,-1},{1,1},{-1,1},{1,-1}},{{-1,0},{1,0},{0,1},{0,-1}},{{-1,1},{1,-1},{1,1},{-1,-1}},
		{{0,1},{0,-1},{1,0},{-1,0}},{{1,1},{-1,-1},{1,-1},{-1,1}},{{1,0},{-1,0},{0,-1},{0,1}},{{1,-1},{-1,1},{-1,-1},{1,1}}	
	};

	if (this->mode != 5 && (this->mode || (this->rotmode != 0 && this->mode == 0))) {

		int angle = 0;
		for (int i = 0; i < 8; i++) {
			int a1 = -22 + (45*i);
			int a2 = 23 + (45*i);
			if (this->ya > a1 && this->ya <= a2) {
				angle = i;
				break;
			}
		}

		int mlt = 1;
		if (world->keys[VK_CONTROL]) mlt *= 2;
		if (world->keys[VK_SHIFT]) mlt *= 5;

		if ( world->keys[VK_UP] && !world->keyp[VK_UP] )	{
			this->lx += mlt * xy_mod[angle][0][0];
			this->ly += mlt * xy_mod[angle][0][1];
			world->keyp[VK_UP] = MOVE_DELAY;
		}
		if (world->keyp[VK_UP]) {
			if (world->keys[VK_UP]) {
				world->keyp[VK_UP] --;
			} else {
				world->keyp[VK_UP] = 0;
			}
		}

		if ( world->keys[VK_DOWN] && !world->keyp[VK_DOWN] )	{
			this->lx += mlt * xy_mod[angle][1][0];
			this->ly += mlt * xy_mod[angle][1][1];
			world->keyp[VK_DOWN] = MOVE_DELAY;
		}
		if (world->keyp[VK_DOWN]) {
			if (world->keys[VK_DOWN]) {
				world->keyp[VK_DOWN] --;
			} else {
				world->keyp[VK_DOWN] = 0;
			}
		}

		if ( world->keys[VK_LEFT] && !world->keyp[VK_LEFT] )	{
			this->lx += mlt * xy_mod[angle][2][0];
			this->ly += mlt * xy_mod[angle][2][1];
			world->keyp[VK_LEFT] = MOVE_DELAY;
		}
		if (world->keyp[VK_LEFT]) {
			if (world->keys[VK_LEFT]) {
				world->keyp[VK_LEFT] --;
			} else {
				world->keyp[VK_LEFT] = 0;
			}
		}

		if ( world->keys[VK_RIGHT] && !world->keyp[VK_RIGHT] )	{
			this->lx += mlt * xy_mod[angle][3][0];
			this->ly += mlt * xy_mod[angle][3][1];
			world->keyp[VK_RIGHT] = MOVE_DELAY;
		}
		if (world->keyp[VK_RIGHT]) {
			if (world->keys[VK_RIGHT]) {
				world->keyp[VK_RIGHT] --;
			} else {
				world->keyp[VK_RIGHT] = 0;
			}
		}
	
		if (this->lx < 0) this->lx = 0;
		if (this->ly < 0) this->ly = 0;
		if (this->lx >= this->map_y) this->lx = this->map_y - 1;
		if (this->ly >= this->map_x) this->ly = this->map_x - 1;

	}

	//
	// Mode :: 1-4 :: Start
	//

	float mx = this->lx * CELL_SIZE;
	float mz = this->ly * CELL_SIZE;

	if (abs(mx - this->x) < (MAX_SPEED / 10.0)) {
		this->x = mx;
	} else {
		this->x += (mx - this->x) / 5.0;
	}

	if (abs(mz - this->z) < (MAX_SPEED / 10.0)) {
		this->z = mz;
	} else {
		this->z += (mz - this->z) / 5.0;
	}

//	this->y = this->get_ground_height(this->z, this->z);

	//
	// Mode :: 1-4 :: End
	//

	//
	// Mode :: 0 :: Start
	//

	// Rotate Inertia - X
	if ( !world->keys[VK_UP] && !world->keys[VK_DOWN] || this->rotmode) {
		if (this->xas > (XROTATE_SPEED / 20.0f)) {
			this->xas -= (XROTATE_SPEED / 20.0f);
		} else if (this->xas < -(XROTATE_SPEED / 20.0f)) {
			this->xas += (XROTATE_SPEED / 20.0f);
		} else {
			this->xas = 0.0f;
		}
	}		

	// Rotate Inertia - Y
	if (!world->keys[VK_RIGHT] && !world->keys[VK_LEFT] || this->rotmode) {
		if (this->yas > (YROTATE_SPEED / 20.0f)) {
			this->yas -= (YROTATE_SPEED / 20.0f);
		} else if (this->yas < -(YROTATE_SPEED / 20.0f)) {
			this->yas += (YROTATE_SPEED / 20.0f);
		} else {
			this->yas = 0.0f;
		}
	}

	//
	// Mode :: 1 :: End
	//

	if (this->cmdr) {
		this->cmod -= 0.04;
		if (this->cmod < 0.0) {
			this->cmod = 0.0;
			this->cmdr = false;
		}
	} else {
		this->cmod += 0.04;
		if (this->cmod > 0.5) {
			this->cmod = 0.5;
			this->cmdr = true;
		}
	}

	// Controls

	if (this->x < 0.0) this->x = 0.0;
	if (this->z < 0.0) this->z = 0.0;
	if (this->x > (this->map_x * CELL_SIZE)) this->x = (float) this->map_x * CELL_SIZE;
	if (this->z > (this->map_y * CELL_SIZE)) this->z = (float) this->map_y * CELL_SIZE;

	this->xa += this->xas;
	this->ya += this->yas;

	if (this->ya < 0.0)		this->ya += 360.0;
	if (this->ya > 360.0)	this->ya -= 360.0;
	if (this->xa > 90.0)	this->xa =  90.0;
	if (this->xa < 15.0)	this->xa =  15.0;

	this->cx = this->x + this->camera * sin (this->wg2r(this->ya)) * cos (this->wg2r(this->xa));
	this->cy = this->y + this->camera * sin (this->wg2r(this->xa));
	this->cz = this->z + this->camera * cos (this->wg2r(this->ya)) * cos (this->wg2r(this->xa));


}

void c_editor::render (void* vworld) {

	c_world_p world = (c_world_p) vworld;

	// Look At Viewpoint
	gluLookAt (this->cx, this->cy, this->cz, this->x, this->y, this->z, 0, 1, 0);

	float tx[2][4][4] = {
		{ {0.0, 0.0, 1.0, 1.0}, {1.0, 0.0, 0.0, 1.0}, {1.0, 1.0, 0.0, 0.0}, {0.0, 1.0, 1.0, 0.0} },
		{ {0.0, 1.0, 1.0, 0.0}, {0.0, 0.0, 1.0, 1.0}, {1.0, 0.0, 0.0, 1.0}, {1.0, 1.0, 0.0, 0.0} }
	};

	glColor3f (1.0, 1.0, 1.0);

	for (int i = 0; i < this->map_x; i++) {
		for (int j = 0; j < this->map_y; j++) {

			float x1 = CELL_SIZE * i;
			float x2 = CELL_SIZE * (i + 1);
			float z1 = CELL_SIZE * j;
			float z2 = CELL_SIZE * (j + 1);

			int	i2 = min(i+1, this->map_x-1);
			int	j2 = min(j+1, this->map_y-1);

			float y11 = this->get_map_height (i,  j);
			float y21 = this->get_map_height (i2, j);
			float y22 = this->get_map_height (i2, j2);
			float y12 = this->get_map_height (i,  j2);

			if (this->lx == i && this->ly == j) {
				switch (this->mode) {

					case 0: 
					case 2:
					case 4:
						glColor3f(0.5+this->cmod, 0.5+this->cmod, 0.5+this->cmod);
						break;

					case 1: 
						if (this->get_map_land(i, j)) {
							glColor3f(0.5+this->cmod, 0.5+this->cmod, 0.5+this->cmod);
						} else {
							glColor3f(0.1+this->cmod, 0.1+this->cmod, 0.1+this->cmod);
						}
						break;

					case 3: 
						if (this->get_map_recource(i, j)) {
							glColor3f(0.5+this->cmod, this->cmod, this->cmod);
						} else {
							glColor3f(0.5+this->cmod, 0.5+this->cmod, 0.5+this->cmod);
						}
						break;

				}
			} else {
				switch (this->mode) {

					case 0: 
					case 2:
					case 4:
						glColor3f(1.0, 1.0, 1.0);
						break;

					case 1: 
						if (this->get_map_land(i, j)) {
							glColor3f(1.0, 1.0, 1.0);
						} else {
							glColor3f(0.6, 0.6, 0.6);
						}
						break;

					case 3: 
						if (this->get_map_recource(i, j)) {
							glColor3f(1.0, 0.5, 0.5);
						} else {
							glColor3f(1.0, 1.0, 1.0);
						}
						break;
				}

			}

			int tx_sm = this->get_map_texture(i, j);
			int tx_id = map_tex_id (tx_sm);
			int tx_rt = map_tex_rt (tx_sm);

			glBindTexture(GL_TEXTURE_2D, this->textures[tx_id]);			

			glBegin (GL_QUADS);
				glTexCoord2f(tx[0][0][tx_rt], tx[1][0][tx_rt]);		glVertex3f(x1, y11, z1);
				glTexCoord2f(tx[0][1][tx_rt], tx[1][1][tx_rt]);		glVertex3f(x1, y12, z2);
				glTexCoord2f(tx[0][2][tx_rt], tx[1][2][tx_rt]);		glVertex3f(x2, y22, z2);
				glTexCoord2f(tx[0][3][tx_rt], tx[1][3][tx_rt]);		glVertex3f(x2, y21, z1);
			glEnd ();

		}
	}
	
	glBindTexture(GL_TEXTURE_2D, 0);

}