#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <windows.h>
#include <gl\gl.h>                      
#include <gl\glu.h>  
#include "map.h"

c_map::c_map () { }

c_map::~c_map () {

	this->save_map ();

	free (this->data);
	glDeleteTextures (LAND_TEXTURES, &this->textures[0]);

}

void c_map::create (void* vworld, int x, int y) {

	int size = x * y;

	this->map_x = x;
	this->map_y = y;

	this->data		= (map_point_p) malloc(size * sizeof (map_point_t));

	if (! this->load_map() ) {
		this->generate_map ();
	}

	this->load_texture ("data/map.bmp");

}

void c_map::render (void* vworld) {

	float tx[2][4][4] = {
		{ {0.0, 0.0, 1.0, 1.0}, {1.0, 0.0, 0.0, 1.0}, {1.0, 1.0, 0.0, 0.0}, {0.0, 1.0, 1.0, 0.0} },
		{ {0.0, 1.0, 1.0, 0.0}, {0.0, 0.0, 1.0, 1.0}, {1.0, 0.0, 0.0, 1.0}, {1.0, 1.0, 0.0, 0.0} }
	};

	glColor3f (1.0, 1.0, 1.0);

	for (int i = 0; i < this->map_x; i++) {
		for (int j = 0; j < this->map_y; j++) {

			float x1 = CELL_SIZE * i;
			float x2 = CELL_SIZE * (i + 1);
			float z1 = CELL_SIZE * j;
			float z2 = CELL_SIZE * (j + 1);

			int	i2 = min(i+1, this->map_x-1);
			int	j2 = min(j+1, this->map_y-1);

			float y11 = this->get_map_height (i,  j);
			float y21 = this->get_map_height (i2, j);
			float y22 = this->get_map_height (i2, j2);
			float y12 = this->get_map_height (i,  j2);

			if (this->get_map_land(i, j) == 0) {
				glColor3f (0.4, 0.4, 0.4);				
			} else if (! this->is_free(i, j)) {	
				glColor3f (0.4, 1.0, 0.4);
			} else if (this->get_map_recource(i, j)) {	
				glColor3f (1.0, 0.4, 0.4);
			} else {
				glColor3f (1.0, 1.0, 1.0);
			}

			int tx_sm = this->get_map_texture(i, j);
			int tx_id = map_tex_id (tx_sm);
			int tx_rt = map_tex_rt (tx_sm);

			glBindTexture(GL_TEXTURE_2D, this->textures[tx_id]);			

			glBegin (GL_QUADS);
				glTexCoord2f(tx[0][0][tx_rt], tx[1][0][tx_rt]);		glVertex3f(x1, y11, z1);
				glTexCoord2f(tx[0][1][tx_rt], tx[1][1][tx_rt]);		glVertex3f(x1, y12, z2);
				glTexCoord2f(tx[0][2][tx_rt], tx[1][2][tx_rt]);		glVertex3f(x2, y22, z2);
				glTexCoord2f(tx[0][3][tx_rt], tx[1][3][tx_rt]);		glVertex3f(x2, y21, z1);
			glEnd ();

			glBindTexture(GL_TEXTURE_2D, 0);
			glPointSize(2.0);
			glBegin (GL_POINTS);
				glVertex3f(x1, y11, z1);
			glEnd();


		}
	}
	
	glBindTexture(GL_TEXTURE_2D, 0);

}

int c_map::generate_map () {

	int size = this->map_x * this->map_y;

	for (int i = 0; i < size; i++) {
		this->data[i].height	= 0;
		this->data[i].land		= 1;
		this->data[i].object	= 0;
		this->data[i].recource	= 0;
		this->data[i].texture	= 0;
	}

	return true;

}

int c_map::validate (int x, int y) {

	if (x < 0) return false;
	if (x >= this->map_x) return false;
	if (y < 0) return false;
	if (y >= this->map_y) return false;

	return true;

}

int c_map::coordinate (int x, int y) {

	if (this->validate(x, y)) {
		return (x * this->map_y) + y;
	} else {
		return -1;
	}

}

int c_map::is_free (int x, int y) {
	
	int i = this->coordinate (x, y);
	if (i == -1) return false;

	return ( this->data[i].land && (! this->data[i].object) );

}

//
// Get Functions
//

int c_map::get_map_data (int x, int y, map_point_p point) {

	int i = this->coordinate (x, y);
	if (i == -1) return false;

	*point = this->data[i];
	return true;

}

int c_map::get_map_land (int x, int y) {

	int i = this->coordinate (x, y);
	if (i == -1) return -1;

	return this->data[i].land;

}

int c_map::get_map_recource (int x, int y) {

	int i = this->coordinate (x, y);
	if (i == -1) return -1;

	return this->data[i].recource;

}

int c_map::get_map_texture (int x, int y) {

	int i = this->coordinate (x, y);
	if (i == -1) return -1;

	return this->data[i].texture;

}

int c_map::get_map_object (int x, int y) {

	int i = this->coordinate (x, y);
	if (i == -1) return -1;

	return this->data[i].object;

}

int c_map::get_map_height_i (int x, int y) {

	int i = this->coordinate (x, y);
	if (i == -1) return -1;

	return this->data[i].height;

}

float c_map::get_map_height (int x, int y) {

	int i = this->coordinate (x, y);
	if (i == -1) return -1;

	return (this->data[i].height / 255.0) * MAP_HEIGHT;

}

//
// Set Functions
//

int c_map::set_map_data (int x, int y, map_point_t d) {

	int i = this->coordinate (x, y);
	if (i == -1) return -1;

	this->data[i] = d;

	return true;

}

int c_map::set_map_land (int x, int y, int l) {

	int i = this->coordinate (x, y);
	if (i == -1) return -1;

	this->data[i].land = l;

	return true;

}

int c_map::set_map_recource (int x, int y, int r) {

	int i = this->coordinate (x, y);
	if (i == -1) return -1;

	this->data[i].recource = r;

	return true;

}

int c_map::set_map_texture (int x, int y, int tx_id, int tx_rt) {

	int i = this->coordinate (x, y);
	if (i == -1) return -1;

	this->data[i].texture = tx_id | ( tx_rt << 6);

	return true;

}

int c_map::set_map_object (int x, int y, int id) {

	int i = this->coordinate (x, y);
	if (i == -1) return -1;

	this->data[i].object = id;

	return true;

}

int c_map::set_map_height_i (int x, int y, int h) {

	int i = this->coordinate (x, y);
	if (i == -1) return -1;

	if (h < 0) h = 0;
	if (h > 255) h = 255;

	this->data[i].height = h;

	return true;

}

int c_map::distance (int x1, int y1, int x2, int y2) {
	return abs(x1 - x2) + abs(y1 - y2) - 1;
}

int c_map::nearest_free(int *x, int *y) {

	return true;

}

int c_map::resize (int new_x, int new_y) {

	map_point_p new_data;
	map_point_t d;

	int size = new_x * new_y;
	new_data = (map_point_p) malloc (sizeof(map_point_t)*size);

	for (int i = 0; i < size; i++) {
		new_data[i].height	= 0;
		new_data[i].land		= 1;
		new_data[i].object	= 0;
		new_data[i].recource	= 0;
		new_data[i].texture	= 0;
	}

	for (int i = 0; i < min(this->map_x, new_x); i++) {
		for (int j = 0; j < min(this->map_y, new_y); j++) {
			
			this->get_map_data (i, j, &d);
			int id = (i * new_y) + j;
			new_data[id] = d;

		}
	}

	free (this->data);
	this->data = new_data;
	this->map_x = new_x;
	this->map_y = new_y;

	return true;

}

int c_map::load_map () {

	FILE *input;

	input = fopen ("map.save", "r");
	if (input == 0) return false;

	free (this->data);

	fread (&this->map_x, sizeof(int), 1, input);
	fread (&this->map_y, sizeof(int), 1, input);

	int size = this->map_x * this->map_y;

	this->data		= (map_point_p) malloc(size * sizeof (map_point_t));

	fread (this->data, sizeof(map_point_t), size, input);

/*	for (int i = 0; i < size; i++) {
		fread (this->data, sizeof(map_point_t), size, input);
	}*/

	for (int i = 0; i < size; i++) {
		this->data[i].object = 0;
	}

	fclose (input);

	return true;

}

int c_map::save_map () {

	FILE *output;

	output = fopen ("map.save", "w");
	if (output == 0) return false;

	fwrite (&this->map_x, sizeof(int), 1, output);
	fwrite (&this->map_y, sizeof(int), 1, output);

	int size = this->map_x * this->map_y;

	fwrite (this->data, sizeof(map_point_t), size, output);
/*
	for (int i = 0; i < size; i++) {
		fwrite (this->data, sizeof(map_point_t), size, output);
	}
*/
	fclose (output);

	return true;

}

int c_map::load_texture (char* filename) {

	FILE				*fp;   
	BITMAPFILEHEADER	header;  
	BITMAPINFO			info;
	byte				*data, *bytes, *seekptr;
	size_t				datasize;
	int					i, j;

	// Open Bitmap File
	fp = fopen(filename, "rb");
	if (fp == NULL) {
		MessageBox (0, filename, "File Open Fail", MB_OK);
		return false;
	}

	// Read Header
	if (fread(&header, sizeof(BITMAPFILEHEADER), 1, fp) < 1) {
		fclose(fp);
		MessageBox (0, "Can't read header", "File Error", MB_OK);
		return false;
	}

	// Load Only Bitmap
	if (header.bfType != 'MB')  {
		fclose(fp);
		MessageBox (0, "Wrong header", "File Error", MB_OK);
		return false;
	}

	// Read Bitmap Header
	if (fread(&info, sizeof(BITMAPINFO), 1, fp) < 1) {
	    fclose(fp);
		MessageBox (0, "Wrong Bitmap Header", "File Error", MB_OK);
		return false;
	}

	// Allocate Memory for Bitmap Data
	datasize = info.bmiHeader.biWidth * info.bmiHeader.biHeight * 4;
	data	= (byte*) malloc (datasize);
	bytes	= (byte*) malloc (3);
	seekptr	= data;

	fseek (fp, header.bfOffBits, SEEK_SET);

	// Read Bytes of Bitmap
	for (i = 0; i < info.bmiHeader.biWidth; i++) {
		for (j = 0; j < info.bmiHeader.biHeight; j++) {
			// Reading 3 bytes from file
			fread (bytes, 3, 1, fp);
			seekptr[0] = bytes[2];	// R
			seekptr[1] = bytes[1];	// G
			seekptr[2] = bytes[0];	// B
			seekptr[3] = 0xff;
			// Going 4 bytes (RGBA) forward
			seekptr += 4;
		}
	}

	// Bitmap is not more needed
	fclose(fp);

	int		tx_width	= info.bmiHeader.biWidth	/ LAND_TEXROW;
	int		tx_height	= info.bmiHeader.biHeight	/ LAND_TEXROW;
	long	size		= tx_height * tx_width * 4;

	byte	*smalltex;
	smalltex = (byte*) malloc(size);

	glGenTextures (LAND_TEXTURES, &this->textures[0]);

	for (int i = 0; i < LAND_TEXROW; i++) {
		for (int j = 0; j < LAND_TEXROW; j++) {

			int tx_id = (i * LAND_TEXROW) + j;

			for (int y = 0; y < tx_height; y++) {
				for (int x = 0; x < tx_width; x++) {

					long id1 = ( ((i * tx_height + y) * info.bmiHeader.biWidth) + (j * tx_width) + x ) * 4;
					long id2 = ((y * tx_width) + x) * 4;

					smalltex[id2]	= data[id1];
					smalltex[id2+1]	= data[id1+1];
					smalltex[id2+2]	= data[id1+2];
					smalltex[id2+3]	= data[id1+3];

				}
			}

			// OpenGL Texture Generation
			glBindTexture(GL_TEXTURE_2D, this->textures[tx_id]);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
			glTexImage2D(GL_TEXTURE_2D, 0, 3, tx_width, tx_height, 0, GL_RGBA, GL_UNSIGNED_BYTE, smalltex);

		}
	}


	// Free Memory of Bitmap
	free (smalltex);	
	free (data);
	free (bytes);
  
	return true;

}