#pragma once

#define LAND_ERROR		-1
#define LAND_CLOSE		0

// Textures
#define	map_tex_id(i)	((i) & 0x3f)
#define	map_tex_rt(i)	(((i) & 0xc0) >> 6)

#define LAND_R1			0x00
#define LAND_R2			0x40
#define LAND_R3			0x80
#define LAND_R4			0xc0

#define LAND_TEXROW		8
#define LAND_TEXTURES	64

#define CELL_SIZE		3.0
#define MAP_HEIGHT		10.0

typedef struct map_point {
	unsigned char	land;
	unsigned char	recource;
	unsigned char	texture;
	unsigned char	height;
	int				object;
} map_point_t, *map_point_p;

typedef class c_map {

public:

	int				map_x, map_y;

	c_map ();
	~c_map ();

	// Map Basic Loading
	void			create				(void* vworld, int x, int y);
	void			render				(void* vworld);
	int				generate_map		();
	int				save_map			();
	int				load_map			();
	int				resize				(int new_x, int new_y);

	// Map Data Get
	int				get_map_data		(int x, int y, map_point_p d);
	int				get_map_land		(int x, int y);
	int				get_map_recource	(int x, int y);
	int				get_map_texture		(int x, int y);
	int				get_map_object		(int x, int y);
	int				get_map_height_i	(int x, int y);

	// Map Modification
	int				set_map_land		(int x, int y, int l);
	int				set_map_recource	(int x, int y, int r);
	int				set_map_texture		(int x, int y, int tx_id, int tx_rt);
	int				set_map_object		(int x, int y, int id);		
	int				set_map_height_i	(int x, int y, int h);
	int				set_map_data		(int x, int y, map_point_t d);

	// Cell Validation
	int				nearest_free		(int *x, int *y);
	int				is_free				(int x, int y);
	int				validate			(int x, int y);
	
	// Heights
	float			get_map_height		(int x, int y);
	float			get_ground_height	(float x, float y);

	// Common
	int				distance			(int x1, int y1, int x2, int y2);

protected:

	int				coordinate			(int x, int y);
	int				load_texture		(char *filename);

	map_point_p		data;

	unsigned int	textures[LAND_TEXTURES];

} c_map_t, *c_map_p;
