// Battle Of The Spheres
// Project started: 27 feb 2009 14:12

#pragma comment(lib, "opengl32.lib")
#pragma comment(lib, "glu32.lib")

#include "game/world.h"

#include <windows.h>                
#include <math.h>
#include <gl\gl.h>                      
#include <gl\glu.h>                    


HDC    hDC = NULL;      // GDI Context
HGLRC  hRC = NULL;      // Render Context
HWND   hWnd = NULL;     // Window Descriptor
HINSTANCE hInstance;    // App Descriptor

c_world_p world;		// global context

float fogColor[4] = {0.0f, 0.0f, 0.0f, 0.2f};

void init_gl_scene (GLsizei Width, GLsizei Height) {
	
	srand((int) GetTickCount());

	glEnable(GL_TEXTURE_2D);      // Enable Global Texturing
	glShadeModel(GL_SMOOTH);      // Enable Smooth Shading

	glClearColor(0.0f, 0.0f, 0.0f, 1.0f); // Background color
	glClearDepth(1.0f);       
	glEnable(GL_DEPTH_TEST);     
	glDepthFunc(GL_LESS);     
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST); 

	world->create ();

}

void resize_gl_scene (GLsizei Width, GLsizei Height) {

	if (Height==0)  Height=1;			// AntiDivByZero
	
	glViewport(0, 0, Width, Height);	// Reset Perspective

	glMatrixMode(GL_PROJECTION);        // Reset projection maxtix
    glLoadIdentity();                   // Reset projection maxtix

    // Counting window size
	gluPerspective (45.0f, (GLfloat) Width / (GLfloat) Height, 0.0f, 100.0f);
    glMatrixMode(GL_MODELVIEW);			// Choosing model viewing matrix

	world->win_cx = world->win_x + (world->screen_x / 2);
	world->win_cy = world->win_y + (world->screen_y / 2);

}

void draw_gl_scene () {

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// "cls"
    glLoadIdentity();															// Reset view

	world->render ();

}

void free_gl_scene () {

	ChangeDisplaySettings(NULL, 0);

	wglMakeCurrent(hDC,NULL);
	wglDeleteContext(hRC);
	ReleaseDC(hWnd,hDC);

	world->destroy ();

	// Appocalipsis ;)
	delete world;

}

LRESULT CALLBACK WndProc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {

	RECT    Screen;         //     
	GLuint  PixelFormat;
	static  PIXELFORMATDESCRIPTOR pfd =	{
		sizeof(PIXELFORMATDESCRIPTOR),  //   
        1,                              //   (?)
        PFD_DRAW_TO_WINDOW |            //   
        PFD_SUPPORT_OPENGL |            //   OpenGL
		PFD_GENERIC_ACCELERATED |		// -  ,  ,      ...
		PFD_DOUBLEBUFFER,               //    
        PFD_TYPE_RGBA,                  //  RGBA 
        32,                             //  16   
        0, 0, 0, 0, 0, 0,               //    (?)
        0,                              //   
        0,                              //    (?)
        0,                              //   
        0, 0, 0, 0,                     //    (?)
        32,                             // 16  Z- ( ) 
        0,                              //   
        0,                              //    (?)
        PFD_MAIN_PLANE,                 //   
        0,                              //  (?)
        0, 0, 0                         //    (?)
	};

	switch (message)        // Main Switch
	{

		// App Create
        case WM_CREATE:

			// WM_Create :: Start

			hDC = GetDC (hWnd);      
			PixelFormat = ChoosePixelFormat(hDC, &pfd);
			if (!PixelFormat){
				MessageBox(0, "Can't Find A Suitable PixelFormat.", "Error", MB_OK | MB_ICONERROR);
				PostQuitMessage(0);
                break; 
            }

			SetPixelFormat (hDC, PixelFormat, &pfd);

			hRC = wglCreateContext (hDC);
			if(!hRC) {
                MessageBox(0, "Can't Create A GL Rendering Context.", "Error", MB_OK | MB_ICONERROR);
                PostQuitMessage(0);
                break;
            }

			if(!wglMakeCurrent(hDC, hRC)) {
                MessageBox(0, "Can't activate GLRC.", "Error", MB_OK | MB_ICONERROR);
                PostQuitMessage(0);
                break;
            }

			GetClientRect(hWnd, &Screen);
			init_gl_scene (Screen.right, Screen.bottom);

			break;
			// WM_Create :: End

		// App Die
		case WM_DESTROY:
		case WM_CLOSE:

			// WM_Close :: Start
			free_gl_scene ();
			PostQuitMessage(0);
			break;
			// WM_Close :: End

		// App KeyState

		case WM_KEYDOWN:
			world->keys[wParam] = true;
			break;

		case WM_KEYUP:
			world->keys[wParam] = false;
			break;

		case WM_MOUSEMOVE:
			world->mmx	= world->win_cx - LOWORD(lParam) - world->win_x;
			world->mmy	= world->win_cy - HIWORD(lParam) - world->win_y;
			break;

		// Win Resize

		case WM_SIZE:
			resize_gl_scene (LOWORD(lParam),HIWORD(lParam));
			break;

		// Default Win Proc		

        default:
			return (DefWindowProc(hWnd, message, wParam, lParam));
    } // end - Main Switch

	return (0);

}

int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance, LPSTR lpCmdLine,int nCmdShow)
{
	MSG             msg;    //   Windows
	WNDCLASS        wc;		//   Windows    
	HWND            hWnd;   //   

	wc.style                = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wc.lpfnWndProc          = (WNDPROC) WndProc;
	wc.cbClsExtra           = 0;
	wc.cbWndExtra           = 0;
	wc.hInstance            = hInstance;
	wc.hIcon                = NULL;
	wc.hCursor              = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground        = NULL;
	wc.lpszMenuName         = NULL;
	wc.lpszClassName        = "OpenGL WinClass";

	if(!RegisterClass(&wc))	{
		MessageBox (0,"Failed To Register The Window Class.", "Error", MB_OK|MB_ICONERROR);
		return false;
	}

	world = new c_world_t ();

	world->win_x = 0;
	world->win_y = 0;
	world->screen_x = GetSystemMetrics(SM_CXSCREEN);
	world->screen_y = GetSystemMetrics(SM_CYSCREEN);
	world->win_cx = world->win_x + (world->screen_x / 2);
	world->win_cy = world->win_y + (world->screen_y / 2);

	hWnd = CreateWindow (
        "OpenGL WinClass",
        "Battle Of The Spheres - AlterVision Creatives",	//   
        WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
        world->win_x, world->win_y,						//    
        world->screen_x, world->screen_y,				//    
        NULL,
        NULL,
        hInstance,
        NULL
	);

	if(!hWnd) {
		MessageBox(0,"Window Creation Error.", "Error", MB_OK|MB_ICONERROR); 
		return false;
	}

	ShowWindow(hWnd, SW_SHOW);
	UpdateWindow(hWnd);
	SetFocus(hWnd);

	while (1)
	{
		// Sync Rate :: 30 fps :: Get Time
		DWORD start_time = GetTickCount();

		while (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))	{
			if (GetMessage(&msg, NULL, 0, 0)) {
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			} else {
				return true;
			}
		}

		world->process ();

		draw_gl_scene();                        // Go To The Party ))
		SwapBuffers(hDC);                       // Show the Results

		// EscapeKey for Exit Procedure
		if (world->keys[VK_ESCAPE]) SendMessage (hWnd,WM_CLOSE,0,0);    

		// Sync Rate :: 30 fps :: Syncronize;
		while((GetTickCount() - start_time) < 33) Sleep(3);
	}

}

